from twisted.internet.defer import DeferredList, inlineCallbacks
from twisted.internet import reactor
from ensemble.formula.repository import LocalFormulaRepository
from ensemble.environment import config
import logging
import os
import yaml
import zookeeper

log = logging.getLogger('deploy')


def command(options):
    """
    Deploy a formula to Ensemble!
    """
    actions = [deploy(options)]

    return DeferredList(actions).addBoth(lambda result: reactor.stop())


def configure_subparser(subparser):
    """Setup the argument parser"""
    subparser.add_argument("--repo",
                           required=True,
                           help="""Configure the repo directory""")

    subparser.add_argument("formula",
                           help="""Name of the formula to deploy""")


@inlineCallbacks
def deploy(options):
    """Deploy a service
    """
    # using the repo directory attempt to resolve the formula
    repo_dir = options.repo
    repo = LocalFormulaRepository(repo_dir)
    formula = repo.find(options.formula)

    environment = options.environments.get_default()
    provider = environment.get_machine_provider()

    fs_storage = provider.get_file_storage()
    bundle = formula.as_bundle()
    digest = bundle.get_digest()

    remote_key = "%s-%s-%s" % (formula.info.name,
                               formula.info.revision,
                               digest)
    remote_path = (u"formulas/%s" % remote_key).encode("utf-8")

    log.info("remote path %s bundle path %s", remote_path,
                        bundle.path)

    # TODO: check zk for the key before s3 push
    yield fs_storage.put(remote_path, bundle.path)
    zk_client = yield provider.connect()
    yield configure_zookeeper(zk_client, provider, remote_key, formula)
    yield deploy_service(zk_client, remote_key, formula)


@inlineCallbacks
def configure_zookeeper(zk_client, provider, remote_key, formula):
    children = yield zk_client.get_children("/")
    if not "formulas" in  children:
        yield zk_client.create("/formulas")
    zk_formulas = yield zk_client.get_children("/formulas")
    if not remote_key in zk_formulas:
        yield zk_client.create("/formulas/%s" % remote_key)

    yield zk_client.create("/formulas/%s/metadata" % remote_key,
                    open(os.path.join(formula.path,
                                      'metadata.yaml'), 'r').read()
                    )

    if not "services" in  children:
        yield zk_client.create("/services")

    if not "machines" in  children:
        yield zk_client.create("/machines")


@inlineCallbacks
def deploy_service(zk_client, remote_key, formula):
    # TODO: check if the service is already deployed before
    # doing any of this

    services = yield zk_client.get_children("/services")
    service_path = "/services/%s" % (formula.info.name)

    service_config = yaml.dump(dict(
        formula=remote_key,
        ))

    if formula.info.name not in services:
        yield zk_client.create(service_path, service_config)

    service_units_path = yield zk_client.create("%s/units" % service_path)
    service_unit = yield zk_client.create("%s/%s-" % (service_units_path,
                                                       formula.info.name),
                                          flags=zookeeper.SEQUENCE)

    machine_path = yield zk_client.create("/machines/machine-",
                                          flags=zookeeper.SEQUENCE)

    machine_units_path = yield zk_client.create("%s/units" % (machine_path, ))
    service_unit = os.path.split(service_unit)[-1]
    yield zk_client.create("%s/%s" % (machine_units_path, service_unit))

