import os
import hashlib
import zipfile
import tempfile

from .info import FormulaInfo
from .bundle import FormulaBundle


class FormulaDirectory(object):
    """Directory that holds formula content.

    :param path: Path to formula directory

    The directory must contain the following files::

    - ``metadata.yaml``

    """

    def __init__(self, path):
        self.path = path
        self.info = FormulaInfo(os.path.join(path, "metadata.yaml"))

    def __repr__(self):
        return "<%s %r>" % (self.__class__.__name__,
                              self.path)

    @classmethod
    def from_archive(cls, archive_path, directory_path):
        zf = zipfile.ZipFile(archive_path, 'r')
        zf.extractall(directory_path)
        return cls(directory_path)

    def make_archive(self, path):
        """Create archive of directory and write to ``path``.

        :param path: Path to archive

        - build/* - This is used for packing the formula itself and any
                    similar tasks.
        - */.*    - Hidden files are all ignored for now.  This will most
                    likely be changed into a specific ignore list (.bzr, etc)
        """

        zf = zipfile.ZipFile(path, 'w')

        for dirpath, dirnames, filenames in os.walk(self.path):
            relative_path = dirpath[len(self.path) + 1:]
            if relative_path and not self._ignore(relative_path):
                zf.write(dirpath, relative_path)
            for name in filenames:
                archive_name = os.path.join(relative_path, name)
                if not self._ignore(archive_name):
                    real_path = os.path.join(dirpath, name)
                    zf.write(real_path, archive_name)

        zf.close()

    def _ignore(self, path):
        if path == "build" or path.startswith("build/"):
            return True
        if path.startswith('.'):
            return True

    def _get_build_dir(self):
        build_dir = os.path.join(self.path, "build")
        if not os.path.isdir(build_dir):
            os.mkdir(build_dir)
        return build_dir

    def as_bundle(self):
        build_dir = self._get_build_dir()

        canonical_filename = "%s-%d.frml" % \
                             (self.info.name, self.info.revision)
        temporary_filename = tempfile.mktemp(prefix=canonical_filename,
                                             dir=build_dir)

        self.make_archive(temporary_filename)

        digest = hashlib.md5()
        with open(temporary_filename) as formula_file:
            while True:
                data = formula_file.read(8192)
                if not data:
                    break
                digest.update(data)

        final_filename = "%s/%s-%d-%s.frml" % \
            (build_dir, self.info.name, self.info.revision,
             digest.hexdigest())

        os.rename(temporary_filename, final_filename)
        return FormulaBundle(final_filename)
