import os
import hashlib
import inspect
import tempfile

from ensemble.errors import FileNotFound
from ensemble.formula.info import FormulaInfo
from ensemble.formula.directory import FormulaDirectory
from ensemble.formula.bundle import FormulaBundle
from ensemble.formula import tests

from ensemble.formula.tests.test_repository import RepositoryTestBase


sample_directory = os.path.join(
    os.path.dirname(inspect.getabsfile(tests)), "repository", "dummy")


class DirectoryTest(RepositoryTestBase):

    def test_metadata_is_required(self):
        directory = self.makeDir()
        self.assertRaises(FileNotFound, FormulaDirectory, directory)

    def test_info(self):
        directory = FormulaDirectory(sample_directory)
        self.assertTrue(directory.info is not None)
        self.assertTrue(isinstance(directory.info, FormulaInfo))
        self.assertEquals(directory.info.name, "dummy")

    def test_from_archive(self):
        directory = FormulaDirectory(sample_directory)
        temporary = self.makeDir()
        f = tempfile.NamedTemporaryFile(suffix=".frml")

        # make archive from sample directory
        directory.make_archive(f.name)

        # extract into temporary directory
        FormulaDirectory.from_archive(f.name, temporary)

        def relative_path(basepath, *path):
            basepath = os.path.abspath(basepath)
            abspath = os.path.abspath(os.path.join(*path))
            return abspath[len(basepath) + 1:]

        manifest = set()
        for dirpath, dirnames, filenames in os.walk(temporary):
            for filename in filenames:
                manifest.add(relative_path(temporary, dirpath, filename))
            for dirname in dirnames:
                manifest.add(relative_path(temporary, dirpath, dirname))

        self.assertEqual(
            set(manifest),
            set(("metadata.yaml", "src", "src/hello.c", "empty")))

    def test_make_archive(self):
        # make archive from sample directory
        directory = FormulaDirectory(sample_directory)
        f = tempfile.NamedTemporaryFile(suffix=".frml")
        directory.make_archive(f.name)

        # open archive in .zip-format and assert integrity
        from zipfile import ZipFile
        zf = ZipFile(f.name)
        self.assertEqual(zf.testzip(), None)

        # assert included
        included = [info.filename for info in zf.infolist()]
        self.assertEqual(
            set(included),
            set(("metadata.yaml", "empty/", "src/", "src/hello.c")))

    def test_as_bundle(self):
        directory = FormulaDirectory(self.sample_dir1)

        # Ensure build/ doesn't exist. It should be able to create it.
        self.assertFalse(os.path.isdir(os.path.join(directory.path, "build")))

        formula_bundle = directory.as_bundle()
        self.assertEquals(type(formula_bundle), FormulaBundle)

        with open(formula_bundle.path) as formula_file:
            digest = hashlib.md5(formula_file.read())

        self.assertEquals(formula_bundle.info.name, "sample")
        self.assertEquals(formula_bundle.path,
                          directory.path + "/build/sample-1-%s.frml"
                          % digest.hexdigest())
