"""
Functional tests for secure zookeeper connections using an ssh forwarded port.

Requirements for functional test
 - sshd running on localhost
 - zookeeper running on localhost, listening on port 2181
 - user can log into localhost via key authentication
 - ~/.ssh/id_dsa exists and is configured as an authorized key
"""

import os
import pwd
import zookeeper

from txzookeeper.client import ConnectionTimeoutException

from ensemble.lib.testing import TestCase
from ensemble.storage.connection import connect


class ConnectionTest(TestCase):

    def setUp(self):
        super(ConnectionTest, self).setUp()
        self.username = pwd.getpwuid(os.getuid())[0]
        self.log = self.capture_logging("ensemble.storage.connection")
        zookeeper.set_debug_level(0)

    def tearDown(self):
        super(ConnectionTest, self).tearDown()
        zookeeper.set_debug_level(zookeeper.LOG_LEVEL_DEBUG)

    def test_connect(self):
        """
        Forwarding a port spawns an ssh process with port forwarding arguments.
        """
        connect_deferred, tunnel = connect(
            "localhost", remote_user=self.username, timeout=10)

        def validate_connected(client):
            self.assertTrue(client.connected)
            client.close()
            tunnel.loseConnection()

        connect_deferred.addCallback(validate_connected)
        return connect_deferred

    def test_connect_with_key(self):
        """
        A private key path can be specified for authenticating to the localhost
        """
        connect_deferred, tunnel = connect(
            "localhost",
            remote_user=self.username,
            private_key_path="~/.ssh/id_dsa",
            timeout=15)

        def validate_connected(client):
            self.assertTrue(client.connected)
            client.close()
            tunnel.loseConnection()

        connect_deferred.addCallback(validate_connected)
        return connect_deferred

    def test_timeout(self):
        """
        if a connection can not be made before a timeout period, an exception
        is raised.
        """
        connect_deferred, tunnel = connect(
            "localhost", remote_user="rabbit", timeout=1)
        self.failUnlessFailure(connect_deferred, ConnectionTimeoutException)
        connect_deferred.addCallback(lambda f: tunnel.loseConnection())
        return connect_deferred
