"""
Functional test for EC2 Provider.

Requirements for functional test
 - valid amazon credentials present in the environment as
   AWS_SECRET_ACCESS_KEY, AWS_ACCESS_KEY_ID
 - an ssh key (id_dsa, id_rsa, identity) present in ~/.ssh
 - if the key has a password an ssh-agent must be running with
   SSH_AGENT_PID and SSH_AUTH_SOCK set in the environment.

These tests may take several minutes for each test.
"""

import os
import pwd
import sys
import zookeeper

from twisted.internet.defer import inlineCallbacks, Deferred
from ensemble.lib.testing import TestCase

from ensemble.providers.ec2 import MachineProvider
from ensemble.providers.ec2.utils import wait_for_startup
from ensemble.storage.connection import connect


class EC2ProviderTest(TestCase):

    def setUp(self):
        super(EC2ProviderTest, self).setUp()
        self.username = pwd.getpwuid(os.getuid())[0]
        self.log = self.capture_logging("ensemble")
        zookeeper.set_debug_level(0)
        self.provider = MachineProvider(
            {"control-bucket": "ensemble-test-%s" % (self.username)},
            "ec2-functional", "ec2-functional")

    def _filter_instances(self, instances):
        provider_instances = []
        for i in instances:
            if i.instance_state not in ("running", "pending"):
                continue
            if not self.provider.instance_name in i.reservation.groups:
                continue
            provider_instances.append(i)
        return provider_instances

    @inlineCallbacks
    def test_shutdown(self):
        """
        Shutting down the provider, terminates all instances associated to
        the provider instance.
        """
        running = yield self.provider.ec2.describe_instances()
        running_prior = set(self._filter_instances(running))

        result = yield self.provider.shutdown()

        running = yield self.provider.ec2.describe_instances()
        running_now = set(self._filter_instances(running))

        if result:
            shutdown = running_prior-running_now
            self.assertEqual(len(result), len(shutdown))
            for i in shutdown:
                self.assertTrue(i.dns_name in result)

    @inlineCallbacks
    def test_bootstrap_and_connect(self):
        """
        Launching a bootstrap instance, creates an ec2 instance with
        a zookeeper server running on it. This test may take up to 7m
        """

        machines = yield self.provider.bootstrap()
        instances = yield wait_for_startup(self.provider.ec2, machines)

        test_complete = Deferred()

        def verify_running():
            sys.stderr.write("running; ")

            def validate_connected(client, tunnel):
                self.assertTrue(client.connected)
                sys.stderr.write("connected.")
                client.close()
                tunnel.loseConnection()
                test_complete.callback(None)

            def validate_failure(failure):
                sys.stderr.write("failure.")
                test_complete.errback(failure)

            client_deferred, tunnel = connect(instances[0].dns_name, timeout=600)
            client_deferred.addCallback(validate_connected, tunnel)
            client_deferred.addErrback(validate_failure)

        from twisted.internet import reactor
        reactor.callWhenRunning(verify_running)

        yield test_complete

    # set the timeout to something more reasonable for bootstraping
    test_bootstrap_and_connect.timeout = 800
