"""
Ec2 Provider File Storage on S3
"""
import os
import tempfile

from twisted.internet.defer import fail
from txaws.s3.exception import S3Error

from ensemble.errors import FileNotFound


class FileStorage(object):

    def __init__(self, provider):
        self.provider = provider
        self.s3 = self.provider.s3
        self.control_bucket = self.provider.config.get("control-bucket")

    def get(self, name):
        """
        If the file exists in the storage, this method will download the file
        and return a temporary file. If it doesn't exist then a FileNotFound
        failure is returned.

        @return A deferred with path to the locally stored file.
        @type Deferred
        """
        # for now we do the simplest thing and just fetch the file
        # in a single call, s3 limits this to some mb, as we grow
        # to have formulas that might grow to this size, we can
        # revisit fetching in batches (head request for size) and
        # streaming to disk.

        d = self.s3.get_object(self.control_bucket, name)

        def on_content_retrieved(result):
            # store the file to a tempfile and return
            fh = tempfile.NamedTemporaryFile()
            fh.write(result)
            fh.seek(0)
            return fh

        def on_no_file(failure):
            failure.trap(S3Error)
            return fail(FileNotFound("s3://%s/%s" % (
                self.control_bucket, name)))

        d.addCallback(on_content_retrieved)
        d.addErrback(on_no_file)

        return d

    def put(self, name, file_path):
        """
        Transfer and store at file_path under the given name. Returns True
        if successful.

        @return A deferred with a boolean value
        @type Deferred
        """
        if not (os.path.exists(file_path) and os.path.isfile(file_path)):
            return fail(FileNotFound(file_path))
        return self.s3.put_object(
            self.control_bucket, name, open(file_path).read())
