from txaws.ec2.model import Instance
from twisted.internet.defer import succeed
from yaml import dump

from ensemble.lib.testing import TestCase
from ensemble.providers.ec2.tests.common import EC2TestMixin
from ensemble.state.sshclient import SSHClient

from ensemble.errors import NoConnection
from ensemble.providers.ec2.connect import EC2Connect


class EC2ConnectTest(EC2TestMixin, TestCase):

    def test_connect_no_state(self):
        """
        When loading saved state from S3, the provider bootstrap gracefully
        handles the scenario where there is no saved state.
        """
        self.s3.get_object(self.env_name, "ensemble")
        self.mocker.result(succeed(dump([])))
        self.mocker.replay()

        provider = self.get_provider()
        connect = EC2Connect(provider)
        d = connect.run()
        self.failUnlessFailure(d, NoConnection)
        return d

    def test_connect_not_running(self):
        """
        The bootstrap load of the saved instance state, will attempt to
        validate and filter the saved state to only return running instances.
        If there are no running instances it returns False.
        """
        self.s3.get_object(self.env_name, "ensemble")
        self.mocker.result(succeed(dump(
            {"zookeeper_instances": [["i-x", "x1.example.com"]]})))
        self.ec2.describe_instances(*["i-x"])
        self.mocker.result(succeed([]))
        self.mocker.replay()

        provider = self.get_provider()
        connect = EC2Connect(provider)
        d = connect.run()
        self.failUnlessFailure(d, NoConnection)
        return d

    def test_provider_connect(self):
        """
        Test connecting to a running ensemble zookeeper server within ec2.
        """
        server = Instance("i-foobar", "running", dns_name="x1.example.com")
        self.s3.get_object(self.env_name, "ensemble")
        self.mocker.result(succeed(dump(
            {"zookeeper_instances": [
                [server.instance_id, server.dns_name]]})))
        self.ec2.describe_instances(*[server.instance_id])
        self.mocker.result(succeed([server]))

        client = self.mocker.patch(SSHClient)
        client.connect(server.dns_name + ":2181")
        self.mocker.replay()

        provider = self.get_provider()
        d = provider.connect()

        def verify_result(result):
            self.assertTrue(isinstance(result, SSHClient))

        d.addCallback(verify_result)
        return d
