from txaws.ec2.model import Instance, SecurityGroup
from yaml import dump, load
from twisted.internet.defer import succeed
from common import EC2TestMixin

from ensemble.lib.testing import TestCase
from ensemble.lib.mocker import MATCH
from ensemble.machine import ProviderMachine
from ensemble.providers.ec2.launch import EC2Bootstrap
from ensemble.providers.common import (
    DEFAULT_REPOSITORIES, DEFAULT_PACKAGES, BOOTSTRAP_PACKAGES)


class MachineLaunchMixin(object):

    def _mock_launch_utils(self):
        get_public_key = self.mocker.replace(
            "ensemble.providers.common.get_user_public_key")

        def match_config(arg):
            return isinstance(arg, dict)

        get_public_key(MATCH(match_config))
        self.mocker.result("zebra")
        get_ami = self.mocker.replace(
            "ensemble.providers.ec2.utils.get_current_ami")
        get_ami()
        self.mocker.result(succeed("ami-714ba518"))

    def _mock_create_group(self):
        self.ec2.create_security_group(
            self.env_name,
            "Ensemble group for %s" % self.env_name)
        self.mocker.result(succeed(True))

        self.ec2.authorize_security_group(
            self.env_name, ip_protocol="tcp", from_port="22",
            to_port="22", cidr_ip="0.0.0.0/0")
        self.mocker.result(succeed([self.env_name]))

        self.ec2.describe_security_groups(self.env_name)
        self.mocker.result(succeed(
            [SecurityGroup(self.env_name, "", owner_id="123")]))

        self.ec2.authorize_security_group(
            self.env_name, source_group_name=self.env_name,
            source_group_owner_id="123")
        self.mocker.result(succeed(True))


class EC2MachineLaunchTest(EC2TestMixin, MachineLaunchMixin, TestCase):

    def test_provider_launch(self):
        """
        The provider can be used to launch a machine with a minimal set of
        required packages, repositories, and and security groups.
        """
        instance = Instance("i-foobar", "running", dns_name="x1.example.com")

        self.ec2.describe_security_groups()
        self.mocker.result(succeed([]))
        self._mock_create_group()
        self._mock_launch_utils()

        def verify_user_data(data):
            lines = data.split("\n")
            self.assertEqual(lines.pop(0), "#cloud-config")
            config = load("\n".join(lines))
            repos = [dict(source=r) for r in DEFAULT_REPOSITORIES]
            self.assertEqual(config["apt_sources"], repos)
            self.assertEqual(config["packages"], DEFAULT_PACKAGES)
            return True

        self.ec2.run_instances(
            image_id="ami-714ba518",
            instance_type="m1.small",
            max_count=1,
            min_count=1,
            security_groups=[self.env_name],
            user_data=MATCH(verify_user_data))

        self.mocker.result(succeed([instance]))
        self.mocker.replay()

        def verify_result(result):
            self.assertEqual(len(result), 1)
            result = result.pop()
            self.assertTrue(isinstance(result, ProviderMachine))
            self.assertTrue(result.dns_name, instance.dns_name)

        provider = self.get_provider()
        d = provider.start_machine()
        d.addCallback(verify_result)
        return d


class EC2BootstrapTest(EC2TestMixin, MachineLaunchMixin, TestCase):

    def _mock_save(self):
        """
        Mock saving bootstrap instances to S3.
        """
        self.s3.get_bucket(self.env_name)
        self.mocker.result(succeed(True))

        def match_string(data):
            return isinstance(data, str)

        self.s3.put_object(
            self.env_name, "ensemble",
            MATCH(match_string),
            content_type="text/plain")
        self.mocker.result(succeed(True))

    def _mock_launch(self):
        """Mock launching a bootstrap machine on ec2."""

        def verify_user_data(data):
            lines = data.split("\n")
            self.assertEqual(lines.pop(0), "#cloud-config")
            config = load("\n".join(lines))
            repos = [dict(source=r) for r in DEFAULT_REPOSITORIES]
            self.assertEqual(config["apt_sources"], repos)
            self.assertEqual(
                config["packages"],
                list(DEFAULT_PACKAGES) + BOOTSTRAP_PACKAGES)
            return True

        self.ec2.run_instances(
            image_id="ami-714ba518",
            instance_type="m1.small",
            max_count=1,
            min_count=1,
            security_groups=[self.env_name],
            user_data=MATCH(verify_user_data))

    def test_launch_bootstrap(self):
        """
        The provider bootstrap can launch a bootstrap/zookeeper machine.
        """
        self.s3.get_object(self.env_name, "ensemble")
        self.mocker.result(succeed(None))
        self.ec2.describe_security_groups()
        self.mocker.result(succeed([]))
        self._mock_create_group()
        self._mock_launch_utils()
        self._mock_launch()
        self.mocker.result(succeed([]))
        self._mock_save()
        self.mocker.replay()

        provider = self.get_provider()
        bootstrap = EC2Bootstrap(provider)
        return bootstrap.run()

    def test_launch_bootstrap_existing_provider_group(self):
        """
        When launching a bootstrap instance the provider will use an existing
        provider instance group.
        """
        self.s3.get_object(self.env_name, "ensemble")
        self.mocker.result(succeed(None))
        self.ec2.describe_security_groups()
        self.mocker.result(succeed([SecurityGroup(self.env_name, "")]))
        self._mock_launch_utils()
        self._mock_launch()
        self.mocker.result(succeed([]))
        self._mock_save()
        self.mocker.replay()

        provider = self.get_provider()
        bootstrap = EC2Bootstrap(provider)
        return bootstrap.run()

    def test_run_with_loaded_state(self):
        """
        If the provider bootstrap is run when there is already a running
        bootstrap instance, it will just return the existing instance.
        """
        instances = [
            Instance("i-foobar", "running", dns_name="x1.example.com")]
        state = dump({
            "zookeeper_instances":
            [[i.instance_id, i.dns_name] for i in instances]})
        self.s3.get_object(self.env_name, "ensemble")
        self.mocker.result(succeed(state))
        self.ec2.describe_instances(*[i.instance_id for i in instances])
        self.mocker.result(succeed(instances))
        self.mocker.replay()

        def validate_result(result):
            self.assertTrue(result)
            machine = result.pop()
            self.assertTrue(isinstance(machine, ProviderMachine))
            self.assertEqual(machine.dns_name, instances[0].dns_name)

        provider = self.get_provider()
        bootstrap = EC2Bootstrap(provider)
        d = bootstrap.run()
        d.addCallback(validate_result)
        return d

    def test_run_with_launch(self):
        """
        The provider bootstrap will launch an instance when run if there
        is no existing instance.
        """
        instances = [
            Instance("i-foobar", "running", dns_name="x1.example.com")]

        self.s3.get_object(self.env_name, "ensemble")
        self.mocker.result(succeed(None))
        self.ec2.describe_security_groups()
        self.mocker.result(succeed([SecurityGroup(self.env_name, "")]))
        self._mock_launch_utils()
        self._mock_launch()
        self.mocker.result(succeed(instances))
        self._mock_save()
        self.mocker.replay()

        def validate_result(result):
            self.assertEqual(len(result), 1)
            result = result.pop()
            self.assertTrue(isinstance(result, ProviderMachine))
            self.assertTrue(result.dns_name, instances[0].dns_name)

        provider = self.get_provider()
        bootstrap = EC2Bootstrap(provider)
        d = bootstrap.run()
        d.addCallback(validate_result)
        return d

    def test_provider_bootstrap(self):
        """
        The ec2 provider facade instantiates and runs the bootstrap.
        """
        instances = [
            Instance("i-foobar", "running", dns_name="x1.example.com")]
        self.s3.get_object(self.env_name, "ensemble")
        self.mocker.result(succeed(None))
        self.ec2.describe_security_groups()
        self.mocker.result(succeed([SecurityGroup(self.env_name, "")]))
        self._mock_launch_utils()
        self._mock_launch()
        self.mocker.result(succeed(instances))
        self._mock_save()
        self.mocker.replay()

        def validate_result(result):
            self.assertEqual(len(result), 1)
            result = result.pop()
            self.assertTrue(isinstance(result, ProviderMachine))
            self.assertTrue(result.dns_name, instances[0].dns_name)

        provider = self.get_provider()
        d = provider.bootstrap()
        d.addCallback(validate_result)
        return d
